package com.clx.performance.service.impl.collect;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.clx.performance.dao.OrderChildDao;
import com.clx.performance.dao.collect.CollectLineReportDao;
import com.clx.performance.enums.PerformanceResultEnum;
import com.clx.performance.model.OrderChild;
import com.clx.performance.model.collect.CollectLineReport;
import com.clx.performance.param.app.collect.*;
import com.clx.performance.service.collect.CollectLineReportService;
import com.clx.performance.struct.collect.CollectLineReportStruct;
import com.clx.performance.vo.app.collect.CollectLineReportVO;
import com.msl.user.data.UserSessionData;
import com.msl.user.utils.TokenUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Objects;

@Slf4j
@Service
public class CollectLineReportServiceImpl implements CollectLineReportService {

    @Autowired
    private CollectLineReportDao collectLineReportDao;
    @Autowired
    private OrderChildDao orderChildDao;

    @Autowired
    private CollectLineReportStruct collectLineReportStruct;


    @Override
    public void saveSendAddressReport(CollectLineReportSendAddressSaveParam param) {
        UserSessionData loginUserInfo = TokenUtil.getLoginUserInfo();
        Long userNo = loginUserInfo.getUserNo();

        OrderChild orderChild = orderChildDao.getByChildNo(param.getChildNo()).orElseThrow(PerformanceResultEnum.DATA_NOT_FIND);

        CollectLineReport collectLineReport = new CollectLineReport();
        collectLineReport.setUserNo(userNo);
        collectLineReport.setName(loginUserInfo.getUserName());
        collectLineReport.setMobile(loginUserInfo.getUserMobile());
        collectLineReport.setTruckNo(orderChild.getTruckNo());

        collectLineReport.setChildNo(param.getChildNo());
        collectLineReport.setAddressId(orderChild.getSendAddressId());
        collectLineReport.setAddress(orderChild.getSendAddress());

        collectLineReport.setType(CollectLineReport.Type.SEND.getCode());
        collectLineReport.setReportType(CollectLineReport.ReportType.SEND.getCode());

        collectLineReport.setReportStatus(param.getReportStatus());
        collectLineReport.setRemark(param.getStatus()==1? "有存煤" : "现产现装");
        collectLineReport.setReportTime(LocalDateTime.now());
        collectLineReportDao.saveEntity(collectLineReport);
    }

    @Override
    public void saveSendAddressAbnormalReport(CollectLineReportSendAddressAbnormalSaveParam param) {
        UserSessionData loginUserInfo = TokenUtil.getLoginUserInfo();
        Long userNo = loginUserInfo.getUserNo();

        OrderChild orderChild = orderChildDao.getByChildNo(param.getChildNo()).orElseThrow(PerformanceResultEnum.DATA_NOT_FIND);

        CollectLineReport collectLineReport = new CollectLineReport();
        collectLineReport.setUserNo(userNo);
        collectLineReport.setName(loginUserInfo.getUserName());
        collectLineReport.setMobile(loginUserInfo.getUserMobile());
        collectLineReport.setTruckNo(orderChild.getTruckNo());

        collectLineReport.setChildNo(param.getChildNo());
        collectLineReport.setAddressId(orderChild.getSendAddressId());
        collectLineReport.setAddress(orderChild.getSendAddress());

        collectLineReport.setType(CollectLineReport.Type.SEND.getCode());
        collectLineReport.setReportType(CollectLineReport.ReportType.SEND.getCode());

        collectLineReport.setReportStatus(CollectLineReport.ReportStatus.ABNORMAL.getCode());
        collectLineReport.setRemark(param.getRemark());
        collectLineReport.setReportTime(LocalDateTime.now());
        collectLineReportDao.saveEntity(collectLineReport);
    }

    @Override
    public void saveLoadAbnormalReport(CollectLineReportLoadAbnormalSaveParam param) {
        UserSessionData loginUserInfo = TokenUtil.getLoginUserInfo();
        Long userNo = loginUserInfo.getUserNo();

        OrderChild orderChild = orderChildDao.getByChildNo(param.getChildNo()).orElseThrow(PerformanceResultEnum.DATA_NOT_FIND);

        CollectLineReport collectLineReport = new CollectLineReport();
        collectLineReport.setUserNo(userNo);
        collectLineReport.setName(loginUserInfo.getUserName());
        collectLineReport.setMobile(loginUserInfo.getUserMobile());
        collectLineReport.setTruckNo(orderChild.getTruckNo());

        collectLineReport.setChildNo(param.getChildNo());
        collectLineReport.setAddressId(orderChild.getSendAddressId());
        collectLineReport.setAddress(orderChild.getSendAddress());

        collectLineReport.setType(CollectLineReport.Type.SEND.getCode());
        collectLineReport.setReportType(CollectLineReport.ReportType.SEND.getCode());

        collectLineReport.setReportStatus(CollectLineReport.ReportStatus.ABNORMAL.getCode());
        collectLineReport.setRemark(param.getRemark());
        collectLineReport.setReportTime(LocalDateTime.now());
        collectLineReportDao.saveEntity(collectLineReport);
    }

    @Override
    public void saveReceiveAddressAbnormalReport(CollectLineReportReceiveAddressAbnormalSaveParam param) {
        UserSessionData loginUserInfo = TokenUtil.getLoginUserInfo();
        Long userNo = loginUserInfo.getUserNo();

        OrderChild orderChild = orderChildDao.getByChildNo(param.getChildNo()).orElseThrow(PerformanceResultEnum.DATA_NOT_FIND);

        CollectLineReport collectLineReport = new CollectLineReport();
        collectLineReport.setUserNo(userNo);
        collectLineReport.setName(loginUserInfo.getUserName());
        collectLineReport.setMobile(loginUserInfo.getUserMobile());
        collectLineReport.setTruckNo(orderChild.getTruckNo());

        collectLineReport.setChildNo(param.getChildNo());
        collectLineReport.setAddressId(orderChild.getSendAddressId());
        collectLineReport.setAddress(orderChild.getSendAddress());

        collectLineReport.setType(CollectLineReport.Type.RECEIVE.getCode());
        collectLineReport.setReportType(CollectLineReport.ReportType.RECEIVE.getCode());

        collectLineReport.setReportStatus(CollectLineReport.ReportStatus.ABNORMAL.getCode());
        collectLineReport.setRemark(param.getRemark());
        collectLineReport.setReportTime(LocalDateTime.now());
        collectLineReportDao.saveEntity(collectLineReport);
    }

    @Override
    public void saveOrderChildCancelAbnormalReport(CollectLineReportOrderChildCancelAbnormalSaveParam param) {
        UserSessionData loginUserInfo = TokenUtil.getLoginUserInfo();
        Long userNo = loginUserInfo.getUserNo();

        OrderChild orderChild = orderChildDao.getByChildNo(param.getChildNo()).orElseThrow(PerformanceResultEnum.DATA_NOT_FIND);

        CollectLineReport collectLineReport = new CollectLineReport();
        collectLineReport.setUserNo(userNo);
        collectLineReport.setName(loginUserInfo.getUserName());
        collectLineReport.setMobile(loginUserInfo.getUserMobile());
        collectLineReport.setTruckNo(orderChild.getTruckNo());

        collectLineReport.setChildNo(param.getChildNo());
        collectLineReport.setAddressId(orderChild.getSendAddressId());
        collectLineReport.setAddress(orderChild.getSendAddress());

        collectLineReport.setType(CollectLineReport.Type.SEND.getCode());
        collectLineReport.setReportType(CollectLineReport.ReportType.SEND.getCode());

        collectLineReport.setReportStatus(CollectLineReport.ReportStatus.ABNORMAL.getCode());
        collectLineReport.setRemark(param.getRemark());
        collectLineReport.setReportTime(LocalDateTime.now());
        collectLineReportDao.saveEntity(collectLineReport);
    }

    @Override
    public void saveLoadUnloadAbnormalReport(CollectLineReportLoadUnloadAbnormalSaveParam param) {
        UserSessionData loginUserInfo = TokenUtil.getLoginUserInfo();
        Long userNo = loginUserInfo.getUserNo();

        OrderChild orderChild = orderChildDao.getByChildNo(param.getChildNo()).orElseThrow(PerformanceResultEnum.DATA_NOT_FIND);

        CollectLineReport collectLineReport = new CollectLineReport();
        collectLineReport.setUserNo(userNo);
        collectLineReport.setName(loginUserInfo.getUserName());
        collectLineReport.setMobile(loginUserInfo.getUserMobile());
        collectLineReport.setTruckNo(orderChild.getTruckNo());

        collectLineReport.setChildNo(param.getChildNo());
        collectLineReport.setAddressId(orderChild.getSendAddressId());
        collectLineReport.setAddress(orderChild.getSendAddress());

        collectLineReport.setType(Objects.equals(param.getType(),1)? CollectLineReport.Type.SEND.getCode() : CollectLineReport.Type.RECEIVE.getCode());
        collectLineReport.setReportType(Objects.equals(param.getType(),1)? CollectLineReport.ReportType.SEND.getCode() : CollectLineReport.ReportType.RECEIVE.getCode());

        collectLineReport.setReportStatus(CollectLineReport.ReportStatus.ABNORMAL.getCode());
        collectLineReport.setRemark(Objects.equals(param.getType(),1)? "装车时间异常" : "卸车时间异常");
        collectLineReport.setReportTime(LocalDateTime.now());
        collectLineReportDao.saveEntity(collectLineReport);
    }

    @Override
    public void saveOtherReport(CollectLineReportOtherSaveParam param) {
        UserSessionData loginUserInfo = TokenUtil.getLoginUserInfo();
        Long userNo = loginUserInfo.getUserNo();

        CollectLineReport collectLineReport = new CollectLineReport();
        collectLineReport.setUserNo(userNo);
        collectLineReport.setName(loginUserInfo.getUserName());
        collectLineReport.setMobile(loginUserInfo.getUserMobile());

        collectLineReport.setType(CollectLineReport.Type.SEND.getCode());
        collectLineReport.setReportType(CollectLineReport.ReportType.SEND.getCode());

        collectLineReport.setReportStatus(CollectLineReport.ReportStatus.NORMAL.getCode());
        collectLineReport.setRemark(param.getRemark());
        collectLineReport.setReportTime(LocalDateTime.now());
        collectLineReportDao.saveEntity(collectLineReport);
    }

    @Override
    public void saveQualityReport(CollectLineReportQualitySaveParam param) {
        UserSessionData loginUserInfo = TokenUtil.getLoginUserInfo();
        Long userNo = loginUserInfo.getUserNo();

        CollectLineReport collectLineReport = new CollectLineReport();
        collectLineReport.setUserNo(userNo);
        collectLineReport.setName(loginUserInfo.getUserName());
        collectLineReport.setMobile(loginUserInfo.getUserMobile());

        collectLineReport.setType(CollectLineReport.Type.RECEIVE.getCode());
        collectLineReport.setReportType(CollectLineReport.ReportType.RECEIVE.getCode());

        collectLineReport.setReportStatus(CollectLineReport.ReportStatus.NORMAL.getCode());
        collectLineReport.setRemark(param.getRemark());
        collectLineReport.setReportTime(LocalDateTime.now());
        collectLineReportDao.saveEntity(collectLineReport);
    }

    @Override
    public CollectLineReportVO getReportInfo(Integer id) {
        return collectLineReportDao.getEntityByKey(id).map(collectLineReportStruct::convert).orElseThrow(PerformanceResultEnum.DATA_NOT_FIND);

    }

    @Override
    public IPage<CollectLineReportVO> pageLineReport(PageCollectLineReportParam param) {
        UserSessionData loginUserInfo = TokenUtil.getLoginUserInfo();
        Long userNo = loginUserInfo.getUserNo();

        param.setUserNo(userNo);
        return collectLineReportDao.pageLineReport(param);
    }
}
